//Mini CP/M Test by SIGINT
//Using "Multi-Z80 CPU emulator by Neil Bradley (neil@synthcom.com)"


#include<stdio.h>
#include<windows.h>
#include"resource.h"
#include"mz80.h"

HWND mw;
CONTEXTMZ80 cpu;
UINT8 mem[0xffff]; 
OPENFILENAME datei;
HANDLE	hdatei;
HANDLE  disk;
int track,sector,dma;
char dateiname[512];
int datei_gelesen;
BOOL run;
char last_key=0;


int getstring(HANDLE file,void* buffer,int len)
{
	DWORD num;
	int tmp=ReadFile(file,buffer,(DWORD)len,&num,NULL);
	if(tmp==0|num==0){return 0;}else{return 1;}
}


#define TEXT_WINDOW 4096
UCHAR buffer[TEXT_WINDOW];
int line=0;

void clear_text()
{
	*buffer=0;
	SetDlgItemText(mw,ID_TEXT_DIS,"");
}

int print_text(UCHAR *text)
{
	UCHAR *pos;
	int count;
	GetDlgItemText(mw,ID_TEXT_DIS,buffer,TEXT_WINDOW);
	pos=buffer+strlen(buffer);
	while(*text!=0)
	{
		if(!(pos<(buffer+TEXT_WINDOW))){pos=buffer;}
		*pos=*text;
		pos++;
		text++;
		count++;
		if(*(pos-1)=='\n'){line++;}
		if(*(pos-1)==8){pos--;*pos=0;pos--;*pos=0;}
		if(line>15){clear_text();line=0;}
	}

	*pos=0;

	SetDlgItemText(mw,ID_TEXT_DIS,buffer);
	return count;
}


void DisplayWrite(UINT32 adr, UINT8 data, struct MemoryWriteByte *tmp)
{
	UCHAR b[2];
	b[0]=data;
	b[1]=0;
	if(adr==2){print_text(b);}
}

UINT8 KeyRead(UINT32 adr,struct MemoryWriteByte *tmp)
{
	char t;
	switch(adr){

	case 0x00: return (last_key==0)?0x00:0xff; break;
	case 0x01: t=last_key;last_key=0;return (UINT8)t;break;
	}
}

int dma=0;
void DiskWrite(UINT32 adr, UINT8 data, struct MemoryWriteByte *tmp)
{
	int lba=0;
	char buf[128];
	int i;
	DWORD num;

	switch(adr)
	{
	case 16: track=data;break;
	case 18: sector=data;break;
	case 20: dma&=0xff00;dma|=data;break;
	case 21: dma&=0x00ff;dma|=(data<<8);break;
	case 22:
	
		run=FALSE;
		lba=(track*26)+sector;
		SetFilePointer(disk,lba*128,NULL,FILE_BEGIN);
		
		if(data==1){
			getstring(disk,buf,128);
			printf("Read Disk: T:%i S:%i to %x\n",track,sector,dma);
			for(i=0;i<128;i++)
			{
				mem[dma++]=(UINT8)buf[i];
			}
			
		}
		
		if(data==2){
			printf("Write Disk: T:%i S:%i from %x\n",track,sector,dma );
			for(i=0;i<128;i++)
			{
				buf[i]=(char)mem[dma++];
			}
			WriteFile(disk, buf, 128, &num, NULL);
			}
			
			fflush(stdout);
			sector++;
			if(sector>25){sector=0;track++;}
			if(track>76){track=0;}
			run=TRUE;
		break;

	}
}



BOOL CALLBACK DialogProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	UCHAR buf[64];
	switch(uMsg){

	case WM_TIMER:
		if(run)mz80exec(10000);
		break;
	case WM_CHAR:	
		last_key=wParam;
		break;

		
	case WM_COMMAND:
		switch(wParam){
			
		case ID_BUTTON1:	
			if(GetOpenFileName(&datei))
			{
				printf("Loading file:");
				printf(datei.lpstrFile);
				printf("\n");
				fflush(stdout);
				hdatei=CreateFile(datei.lpstrFile,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
				if(hdatei==NULL)(exit -1);
				SetFilePointer(hdatei,0,NULL,FILE_BEGIN);
				getstring(hdatei,buf,2);
				int start=(int)(buf[1]<<8|buf[0]);
				int tmp=start;
				printf("Start: %x\n",start);
				
				while(getstring(hdatei,buf,1)!=0)
				{
					mem[start]=(UINT8)buf[0];
					start++;			
				}
				
				printf("%i Bytes loaded\n",start-tmp);
				fflush(stdout);
				
				CloseHandle(hdatei);
			}
			break;
			
		case ID_BUTTON2: 
			if(run==TRUE){
				SetDlgItemText(mw,ID_BUTTON2,"Run");
				run=FALSE;
			}
			else
			{
				SetDlgItemText(mw,ID_BUTTON2,"Stop");
				
				
				mz80reset();
				run=TRUE;
			}
			break;
		case ID_BUTTON3:
			run=FALSE;
			UINT8 buf[128];
			int i;
			SetFilePointer(disk,0,NULL,FILE_BEGIN);
			getstring(disk,buf,128);

			for(i=0;i<128;i++)
			{
				mem[0x2000+i]=(UINT8)buf[i];
			}
			mz80reset();
			cpu.z80pc=0x2000;
			mz80SetContext(&cpu);
			run=TRUE;
			break;
		default:break;
		}
		break;
	case WM_CLOSE: PostQuitMessage(0);break;

	default: return(0);
	}

}




int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	mw=CreateDialog(hInstance,MAKEINTRESOURCE(ID_DIALOG),NULL,&DialogProc);
	if(mw==NULL){MessageBox(NULL,"Couldn't create Window","ERROR",MB_OK);return 0;}
	ShowWindow(mw,SW_SHOW);

	disk=CreateFile("diskimage",GENERIC_READ|GENERIC_WRITE,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(disk==NULL)(exit -1);
	track=0;
	sector=0;

	print_text("Mini Z80 Test\nby SIGINT\n\nPress Run");
	





	struct z80PortWrite iow[]= {
		{0x02,0x02,DisplayWrite},
		{0x03,0x20,DiskWrite},
		{(UINT32) -1, 	(UINT32) -1, NULL}
	};



	struct z80PortRead ior[]={
		{0x00,0x01,KeyRead},
		{(UINT32) -1,     (UINT32) -1, NULL}
	};


	struct MemoryWriteByte memw[]={
		{(UINT32) -1,     (UINT32) -1, NULL}
	};


	struct MemoryReadByte memr[]={
		{(UINT32) -1,     (UINT32) -1,  NULL}
	};

	cpu.z80MemRead=memr;
	cpu.z80MemWrite=memw;
	cpu.z80IoRead=ior;
	cpu.z80IoWrite=iow;



	MSG message;

	datei.lStructSize=sizeof(OPENFILENAME);
	datei.hwndOwner=mw;
	datei.lpstrFilter="*.rom\0";
	datei.lpstrFile=dateiname;
	datei.nMaxFile=sizeof(dateiname);
	datei.Flags=OFN_FILEMUSTEXIST;


	cpu.z80Base=mem;

	UINT8 buf[128];
	int i;
	getstring(disk,buf,128);

	for(i=0;i<128;i++)
	{
		mem[0x2000+i]=(UINT8)buf[i];
	}

	mz80SetContext(&cpu);

	run=FALSE;

	SetTimer(mw,10006,50,NULL);

	while(GetMessage(&message,NULL,0,0))
	{
		TranslateMessage(&message);
		if(message.message==WM_CHAR){SendMessage(mw,WM_CHAR,message.wParam,message.lParam);continue;}
		if(message.message!=WM_KEYDOWN){DispatchMessage(&message);}
	}

	CloseHandle(disk);
	return message.wParam;

}

